#! /usr/bin/env python
import datetime, MySQLdb, optparse
import yahoofinance
import mdbase


def update_security(security, quotes):
    """Update quotes for a single security in MarketData database."""
    rows = []
    for quote in quotes:
        row = security[0], quote[0], quote[1], quote[2], quote[3], quote[4], quote[5], quote[6]
        rows.append(row)

    db = mdbase.get_db_connection()
    cursor = db.cursor()
    cursor.executemany("INSERT INTO Quotes (idSecurity, date, open, high, low, close, volume, adjustedClose) VALUE (%s, %s, %s, %s, %s, %s, %s, %s)", rows)

    cursor.execute("UPDATE Securities SET lastQuoteDate = %s, markedForUpdate = 0, timeUpdated = %s WHERE idSecurity = %s", (datetime.date.today(), datetime.datetime.now(), security[0]))

    cursor.close()
    db.close()


    
def update_market_data(options):
    """Update MarketData database with quotes downloaded from Yahoo Finance."""

    query = "SELECT idSecurity, ticker, name, lastQuoteDate FROM Securities"
    if options.marked: query += " WHERE markedForUpdate = 1"

    # Open db connection and get all the securities to update
    db = mdbase.get_db_connection()
    cursor = db.cursor()
    cursor.execute(query)
    result = cursor.fetchall()
    cursor.close()
    db.close()

    # Download securities quotes (one day more from the last) and update database
    for record in result:
        if not record[3] is None: 
            lastQuoteDate = record[3] + datetime.timedelta(1) 
        else:
            lastQuoteDate = datetime.date(1900, 1, 1);

        if lastQuoteDate >= datetime.date.today():
            print record[1], "up to date. Skipping"
            continue
        print "Downloading quotes for:", record[1], "from", lastQuoteDate, "to", datetime.date.today()
        quotes = yahoofinance.get_historical_quotes(record[1], lastQuoteDate, datetime.date.today())
        if not quotes is None:
            print "Updating quotes for:", record[1]
            update_security(record, quotes)

#------------------------------------------------------------------------------ 
# Main
#------------------------------------------------------------------------------ 

# Parse command line options
parser = optparse.OptionParser(usage="usage: %prog [options]")
parser.set_defaults(marked=False)
parser.add_option("-m", "--marked", action="store_true", dest="marked", help="Update only marked securities in the database.")
parser.add_option("-a", "--all", action="store_false", dest="marked", help="Update all securities in the database [default].")
(options, args) = parser.parse_args()

# Update MarketData database
update_market_data(options)
